from airflow import DAG      
from airflow.operators.python_operator import PythonOperator, BranchPythonOperator, ShortCircuitOperator  
from airflow.operators.dummy_operator import DummyOperator    
from datetime import datetime  
from data_cleanup import clean_data  
from model_training import train_model    
from data_drift import sample_data  
from prediction import test  
  
def should_retry(**kwargs):    
    # Get the number of retries from XCom    
    retries = kwargs['ti'].xcom_pull(task_ids='high_psi', key='retries') or 0    
    if retries < 3:  # Limit to 3 retries    
        kwargs['ti'].xcom_push(task_ids='high_psi', key='retries', value=retries + 1)    
        return True    
    return False    
  
default_args = {      
    'owner': 'siapa?',      
    'start_date': datetime(2025, 1, 1),  
    'retries': 3      
}      
  
with DAG('customer_churn_pipeline_daily',  
         schedule_interval='@daily',  # Schedule to run daily  
         start_date=datetime(2025, 1, 1),  
         catchup=False,  
         default_args=default_args) as dag:  
      
    clean_data_task = PythonOperator(      
        task_id='clean_data',      
        python_callable=clean_data,      
        op_kwargs={'file_path': 'telco_customer_churn.csv', 'is_daily': True},      
    )      
      
    train_model_task = PythonOperator(      
        task_id='train_model',      
        python_callable=train_model,
        op_kwargs={'is_daily': True},
    )  
  
    generate_drift_task = PythonOperator(  
        task_id='generate_drift',  
        python_callable=sample_data,  
        op_kwargs={  
            'input_csv': 'appended_telco_customer_churn.csv',  
            'output_csv': 'telco_customer_churn_drift.csv',  
            'sample_fraction': 0.2,  
            'random_state': 60  
        }  
    )  
  
    test_model_task = BranchPythonOperator(  
        task_id='test_model',  
        python_callable=test,  
        provide_context=True,
        op_kwargs={'is_daily': True},
    )  
  
    high_psi_task = PythonOperator(  
        task_id='high_psi',  
        python_callable=lambda: print('Handling high psi'),  
        provide_context=True  
    )  
  
    low_psi_task = DummyOperator(  
        task_id='low_psi'  
    )  
      
    # Define task dependencies  
    clean_data_task >> train_model_task >> generate_drift_task >> test_model_task  
    test_model_task >> [low_psi_task, high_psi_task]