from pyspark.sql import SparkSession
from pyspark.ml.feature import StringIndexer, OneHotEncoder, VectorAssembler
from pyspark.ml import Pipeline
from pyspark.sql.functions import col, udf
from pyspark.ml.linalg import SparseVector, DenseVector
from pyspark.sql.types import StringType
import shutil
import os

def clean_data(file_path, is_daily, test_model=False):
    spark = SparkSession.builder.appName("CustomerChurn").getOrCreate()

    # Membaca data dari CSV
    df = spark.read.csv(file_path, header=True, inferSchema=True)
    if (is_daily):
        df_drift = spark.read.csv('telco_customer_churn_drift.csv', header=True, inferSchema=True)
        if "Unnamed: 0" in df_drift.columns and not test_model:  
            df_drift = df_drift.drop("Unnamed: 0")  
        df = df.union(df_drift)
    
    print("Data sebelum pembersihan:")
    df.show()  # Menampilkan data sebelum pembersihan
    print(f"Jumlah baris awal: {df.count()}")

    # Deklarasi kolom numerik dan kategorikal
    numeric_columns = ['SeniorCitizen', 'tenure', 'MonthlyCharges', 'TotalCharges']
    categorical_columns = ['gender', 'Partner', 'Dependents', 'PhoneService',
                           'MultipleLines', 'InternetService', 'OnlineSecurity',
                           'OnlineBackup', 'DeviceProtection', 'TechSupport',
                           'StreamingTV', 'StreamingMovies', 'Contract',
                           'PaperlessBilling', 'PaymentMethod']

    # Mengganti nilai kosong atau spasi menjadi null
    df = df.replace("", None).replace(" ", None)

    # Menghapus baris dengan nilai null di kolom yang relevan
    df_clean = df.na.drop(subset=numeric_columns + categorical_columns)
    print(f"Jumlah baris akhir: {df.count()}")

    # Mengonversi kolom yang seharusnya numerik
    for column in numeric_columns:
        df_clean = df_clean.withColumn(column, col(column).cast('float'))  # Mengonversi ke float

    # Membuat StringIndexer dan OneHotEncoder untuk kolom kategorikal
    indexers = [StringIndexer(inputCol=column, outputCol=column + "_index") for column in categorical_columns]
    encoders = [OneHotEncoder(inputCols=[column + "_index"], outputCols=[column + "_ohe"]) for column in categorical_columns]

    # Menggunakan VectorAssembler untuk mengonversi fitur menjadi vektor
    feature_columns = [column + "_ohe" for column in categorical_columns] + numeric_columns
    assembler = VectorAssembler(inputCols=feature_columns, outputCol="features")

    # Membuat pipeline
    pipeline = Pipeline(stages=indexers + encoders + [assembler])

    # Melatih pipeline untuk mengubah data
    df_transformed = pipeline.fit(df_clean).transform(df_clean)

    # Konversi OneHotEncoded menjadi string
    def vector_to_string(vector):
        if isinstance(vector, SparseVector) or isinstance(vector, DenseVector):
            return ",".join(map(str, vector.toArray()))
        return str(vector)

    vector_to_string_udf = udf(vector_to_string, StringType())

    # Tambahkan kolom string untuk OneHotEncoded
    encoded_columns = [column + "_ohe" for column in categorical_columns]
    for column in encoded_columns:
        df_transformed = df_transformed.withColumn(column + "_str", vector_to_string_udf(col(column)))

    # Drop kolom asli OneHotEncoded
    df_transformed = df_transformed.drop(*encoded_columns)

    # Tambahkan kolom features sebagai string
    df_transformed = df_transformed.withColumn("features_str", vector_to_string_udf(col("features")))

    # Drop kolom 'features' asli
    df_transformed = df_transformed.drop("features")
    df_transformed.show()

    output_path = ""
    # Hapus folder keluaran jika ada
    if (file_path == "telco.customer_churn.csv"):
        output_path = "telco_customer_churn_transformed.csv"
    else:
        output_path = "telco_customer_churn_drift_transformed.csv"
    if os.path.exists(output_path):
        shutil.rmtree(output_path)

    # Simpan data yang telah dibersihkan
    df_transformed.coalesce(1).write.csv(output_path, header=True, mode='overwrite')

    return output_path

if __name__ == "__main__":
    clean_data("telco_customer_churn.csv")